package java.awt.image;

import java.util.Arrays;

public class SinglePixelPackedSampleModel extends SampleModel
{
    private int[] bitMasks;
    private int[] bitOffsets;
    private int[] bitSizes;
    private int maxBitSize;
    private int scanlineStride;
    
    private static native void initIDs();
    
    public SinglePixelPackedSampleModel(int i, int n, int n2, int[] array) {
        this(i, n, n2, n, array);
        if (i != 0 && i != 1 && i != 3) {
            throw new IllegalArgumentException("Unsupported data type " + i);
        }
        return;
    }
    
    public SinglePixelPackedSampleModel(int n, int n2, int n3, int scanlineStride, int[] array) {
        super(n, n2, n3, array.length);
        if (n != 0 && n != 1 && n != 3) {
            throw new IllegalArgumentException("Unsupported data type " + n);
        }
        this.dataType = n;
        this.bitMasks = (int[])(int[])array.clone();
        this.scanlineStride = scanlineStride;
        this.bitOffsets = new int[this.numBands];
        this.bitSizes = new int[this.numBands];
        int n4 = (int)((1L << DataBuffer.getDataTypeSize(n)) - 1L);
        this.maxBitSize = 0;
        for (int i = 0; i < this.numBands; ++i) {
            int n5 = 0;
            int maxBitSize = 0;
            int[] bitMasks = this.bitMasks;
            int n6 = i;
            bitMasks[n6] &= n4;
            int n7 = this.bitMasks[i];
            if (n7 != 0) {
                while ((n7 & 0x1) == 0x0) {
                    n7 >>>= 1;
                    ++n5;
                }
                while ((n7 & 0x1) == 0x1) {
                    n7 >>>= 1;
                    ++maxBitSize;
                }
                if (n7 != 0) {
                    throw new IllegalArgumentException("Mask " + array[i] + " must be contiguous");
                }
            }
            this.bitOffsets[i] = n5;
            if ((this.bitSizes[i] = maxBitSize) > this.maxBitSize) {
                this.maxBitSize = maxBitSize;
            }
        }
        return;
    }
    
    @Override
    public int getNumDataElements() {
        return 1;
    }
    
    private long getBufferSize() {
        return (long)(this.scanlineStride * (this.height - 1) + this.width);
    }
    
    @Override
    public SampleModel createCompatibleSampleModel(int n, int n2) {
        return new SinglePixelPackedSampleModel(this.dataType, n, n2, this.bitMasks);
    }
    
    @Override
    public DataBuffer createDataBuffer() {
        DataBuffer dataBuffer = null;
        int n = (int)this.getBufferSize();
        switch (this.dataType) {
            case 0: {
                dataBuffer = new DataBufferByte(n);
                break;
            }
            case 1: {
                dataBuffer = new DataBufferUShort(n);
                break;
            }
            case 3: {
                dataBuffer = new DataBufferInt(n);
                break;
            }
        }
        return dataBuffer;
    }
    
    @Override
    public int[] getSampleSize() {
        return (int[])this.bitSizes.clone();
    }
    
    @Override
    public int getSampleSize(int n) {
        return this.bitSizes[n];
    }
    
    public int getOffset(int n, int n2) {
        return n2 * this.scanlineStride + n;
    }
    
    public int[] getBitOffsets() {
        return (int[])(int[])this.bitOffsets.clone();
    }
    
    public int[] getBitMasks() {
        return (int[])(int[])this.bitMasks.clone();
    }
    
    public int getScanlineStride() {
        return this.scanlineStride;
    }
    
    @Override
    public SampleModel createSubsetSampleModel(int[] array) {
        if (array.length > this.numBands) {
            throw new RasterFormatException("There are only " + this.numBands + " bands");
        }
        int[] array2 = new int[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = this.bitMasks[array[i]];
        }
        return new SinglePixelPackedSampleModel(this.dataType, this.width, this.height, this.scanlineStride, array2);
    }
    
    @Override
    public Object getDataElements(int n, int n2, Object o, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        switch (this.getTransferType()) {
            case 0: {
                byte[] array;
                if (o == null) {
                    array = new byte[] { 0 };
                }
                else {
                    array = (byte[])(byte[])o;
                }
                array[0] = (byte)dataBuffer.getElem(n2 * this.scanlineStride + n);
                o = array;
                break;
            }
            case 1: {
                short[] array2;
                if (o == null) {
                    array2 = new short[] { 0 };
                }
                else {
                    array2 = (short[])(short[])o;
                }
                array2[0] = (short)dataBuffer.getElem(n2 * this.scanlineStride + n);
                o = array2;
                break;
            }
            case 3: {
                int[] array3;
                if (o == null) {
                    array3 = new int[] { 0 };
                }
                else {
                    array3 = (int[])(int[])o;
                }
                array3[0] = dataBuffer.getElem(n2 * this.scanlineStride + n);
                o = array3;
                break;
            }
        }
        return o;
    }
    
    @Override
    public int[] getPixel(int n, int n2, int[] array, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int[] array2;
        if (array == null) {
            array2 = new int[this.numBands];
        }
        else {
            array2 = array;
        }
        int elem = dataBuffer.getElem(n2 * this.scanlineStride + n);
        for (int i = 0; i < this.numBands; ++i) {
            array2[i] = (elem & this.bitMasks[i]) >>> this.bitOffsets[i];
        }
        return array2;
    }
    
    @Override
    public int[] getPixels(int n, int n2, int n3, int n4, int[] array, DataBuffer dataBuffer) {
        int n5 = n + n3;
        int n6 = n2 + n4;
        if (n < 0 || n >= this.width || n3 > this.width || n5 < 0 || n5 > this.width || n2 < 0 || n2 >= this.height || n4 > this.height || n6 < 0 || n6 > this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array != null) {
        }
        else {
            array = new int[n3 * n4 * this.numBands];
        }
        int n7 = n2 * this.scanlineStride + n;
        int n8 = 0;
        for (int i = 0; i < n4; n7 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; ++j) {
                int elem = dataBuffer.getElem(n7 + j);
                for (int k = 0; k < this.numBands; ++k) {
                    array[n8++] = (elem & this.bitMasks[k]) >>> this.bitOffsets[k];
                }
            }
        }
        return array;
    }
    
    @Override
    public int getSample(int n, int n2, int n3, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        return (dataBuffer.getElem(n2 * this.scanlineStride + n) & this.bitMasks[n3]) >>> this.bitOffsets[n3];
    }
    
    @Override
    public int[] getSamples(int n, int n2, int n3, int n4, int n5, int[] array, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n + n3 > this.width || n2 + n4 > this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array != null) {
        }
        else {
            array = new int[n3 * n4];
        }
        int n6 = n2 * this.scanlineStride + n;
        int n7 = 0;
        for (int i = 0; i < n4; n6 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; ++j) {
                array[n7++] = (dataBuffer.getElem(n6 + j) & this.bitMasks[n5]) >>> this.bitOffsets[n5];
            }
        }
        return array;
    }
    
    @Override
    public void setDataElements(int n, int n2, Object o, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        switch (this.getTransferType()) {
            case 0: {
                dataBuffer.setElem(n2 * this.scanlineStride + n, ((byte[])(byte[])o)[0] & 0xFF);
                break;
            }
            case 1: {
                dataBuffer.setElem(n2 * this.scanlineStride + n, ((short[])(short[])o)[0] & 0xFFFF);
                break;
            }
            case 3: {
                dataBuffer.setElem(n2 * this.scanlineStride + n, ((int[])(int[])o)[0]);
                break;
            }
        }
        return;
    }
    
    @Override
    public void setPixel(int n, int n2, int[] array, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n3 = n2 * this.scanlineStride + n;
        int elem = dataBuffer.getElem(n3);
        for (int i = 0; i < this.numBands; ++i) {
            elem = ((elem & ~this.bitMasks[i]) | (array[i] << this.bitOffsets[i] & this.bitMasks[i]));
        }
        dataBuffer.setElem(n3, elem);
        return;
    }
    
    @Override
    public void setPixels(int n, int n2, int n3, int n4, int[] array, DataBuffer dataBuffer) {
        int n5 = n + n3;
        int n6 = n2 + n4;
        if (n < 0 || n >= this.width || n3 > this.width || n5 < 0 || n5 > this.width || n2 < 0 || n2 >= this.height || n4 > this.height || n6 < 0 || n6 > this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n7 = n2 * this.scanlineStride + n;
        int n8 = 0;
        for (int i = 0; i < n4; n7 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; ++j) {
                int elem = dataBuffer.getElem(n7 + j);
                for (int k = 0; k < this.numBands; ++k) {
                    elem = ((elem & ~this.bitMasks[k]) | (array[n8++] << this.bitOffsets[k] & this.bitMasks[k]));
                }
                dataBuffer.setElem(n7 + j, elem);
            }
        }
        return;
    }
    
    @Override
    public void setSample(int n, int n2, int n3, int n4, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n >= this.width || n2 >= this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        dataBuffer.setElem(n2 * this.scanlineStride + n, (dataBuffer.getElem(n2 * this.scanlineStride + n) & ~this.bitMasks[n3]) | (n4 << this.bitOffsets[n3] & this.bitMasks[n3]));
        return;
    }
    
    @Override
    public void setSamples(int n, int n2, int n3, int n4, int n5, int[] array, DataBuffer dataBuffer) {
        if (n < 0 || n2 < 0 || n + n3 > this.width || n2 + n4 > this.height) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n6 = n2 * this.scanlineStride + n;
        int n7 = 0;
        for (int i = 0; i < n4; n6 += this.scanlineStride, ++i) {
            for (int j = 0; j < n3; ++j) {
                dataBuffer.setElem(n6 + j, (dataBuffer.getElem(n6 + j) & ~this.bitMasks[n5]) | (array[n7++] << this.bitOffsets[n5] & this.bitMasks[n5]));
            }
        }
        return;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null || !(o instanceof SinglePixelPackedSampleModel)) {
            return false;
        }
        SinglePixelPackedSampleModel singlePixelPackedSampleModel = (SinglePixelPackedSampleModel)o;
        return this.width == singlePixelPackedSampleModel.width && this.height == singlePixelPackedSampleModel.height && this.numBands == singlePixelPackedSampleModel.numBands && this.dataType == singlePixelPackedSampleModel.dataType && Arrays.equals(this.bitMasks, singlePixelPackedSampleModel.bitMasks) && Arrays.equals(this.bitOffsets, singlePixelPackedSampleModel.bitOffsets) && Arrays.equals(this.bitSizes, singlePixelPackedSampleModel.bitSizes) && this.maxBitSize == singlePixelPackedSampleModel.maxBitSize && this.scanlineStride == singlePixelPackedSampleModel.scanlineStride;
    }
    
    @Override
    public int hashCode() {
        int n = (((this.width << 8 ^ this.height) << 8 ^ this.numBands) << 8 ^ this.dataType) << 8;
        for (int i = 0; i < this.bitMasks.length; ++i) {
            n = (n ^ this.bitMasks[i]) << 8;
        }
        for (int j = 0; j < this.bitOffsets.length; ++j) {
            n = (n ^ this.bitOffsets[j]) << 8;
        }
        for (int k = 0; k < this.bitSizes.length; ++k) {
            n = (n ^ this.bitSizes[k]) << 8;
        }
        return (n ^ this.maxBitSize) << 8 ^ this.scanlineStride;
    }
    
    static {
        ColorModel.loadLibraries();
        initIDs();
    }
}
